/*!
 * © 2022 David Vines
 *
 * domainOfTheAncients is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * domainOfTheAncients is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with domainOfTheAncients. If not, see https://www.gnu.org/licenses/gpl-2.0.html.
 */
/* global customElements */
/* exported ModalDialog */
class ModalDialog extends HTMLElement {
	constructor() {
		super();
		let oldChildren = [...this.children];

		this.replaceChildren(ModalDialog.wrapperTemplate.content.cloneNode(true));
		this._innerDiv= this.querySelector("div");
		this._innerDiv.append(...oldChildren);

		this._closeButton = this.querySelector(".close-modal-button");
		this._resizeTimer = null;
		this._setStyle();

		this._onclose = null;

		this._closeButton.onclick = this.close.bind(this);
		window.addEventListener("resize", e => this._handleResize() );
	}
	open() {
		this.classList.add("show-modal");
	}
	close() {
		this.classList.remove("show-modal");
		if (this._onclose) {
			this._onclose();
		}
	}
	get onclose() { return this._onclose; }
	set onclose(f) { this._onclose = f; }
	_handleResize() {
		if (this._resizeTimer) clearTimeout(this._resizeTimer);
		this._resizeTimer = setTimeout(() => {
			this._resizeTimer = null;
			this._setStyle();
		});
	}
	_setStyle() {
		let width =  "( " + getComputedStyle(document.querySelector(".post")).marginRight;
			width += " + " + getComputedStyle(document.querySelector(".post")).borderRightWidth;
			width += " + " + getComputedStyle(document.querySelector(".post")).paddingRight;
			width += " + " + getComputedStyle(document.querySelector(".notes")).marginRight;
			width += " + " + getComputedStyle(document.querySelector(".notes")).borderRightWidth;
			width += " + " + getComputedStyle(document.querySelector(".notes")).paddingRight;
			width += " + " + getComputedStyle(document.querySelector(".notes")).width;
			width += " + " + getComputedStyle(document.querySelector(".notes")).paddingLeft;
			width += " + " + getComputedStyle(document.querySelector(".notes")).borderLeftWidth;
			width += " + " + getComputedStyle(document.querySelector(".notes")).marginLeft;
			width += " )";

			this.style.left = `calc( 100vw - ${width} )`;
			this.style.width = `calc( ${width} )`;
	}
}

ModalDialog.wrapperTemplate = document.createElement("template");
ModalDialog.wrapperTemplate.innerHTML = `
<div>
<span class="close-modal-button">&times;</span>
</div>
`;
customElements.define("modal-dialog",ModalDialog);